SET NOCOUNT ON;

;WITH cteRulesStats AS
(
	SELECT	r.*,a.*
	FROM	dbo.ActivityRules r WITH (NOLOCK)
			--LEFT JOIN dbo.ActivityAlerts a WITH (NOLOCK) ON a.RuleID = r.RuleID
			OUTER APPLY 
			(
				SELECT	COUNT(AlertID) AS AlertsCount, 
						COUNT(CASE WHEN a.IsFlagged = 1 THEN AlertID ELSE NULL END) AS AlertsFlaggedCount,
						COUNT(CASE WHEN a.IsAlertSent = 1 THEN AlertID ELSE NULL END) AS AlertsSentCount,
						COUNT(CASE WHEN a.ClientOS = 'Windows' THEN AlertID ELSE NULL END) AS AlertsWinOSCount,
						COUNT(CASE WHEN a.ClientOS = 'Unix' THEN AlertID ELSE NULL END) AS AlertsUnixOSCount,
						COUNT(CASE WHEN a.ClientOS = 'UnKnown' THEN AlertID ELSE NULL END) AS AlertsUKOSCount
						
				FROM	dbo.ActivityAlerts a 
				WHERE	a.RuleID = r.RuleID
			) a
	WHERE	r.IsDeleted = CAST(0 AS BIT)		
)
SELECT	IsActive AS ActiveCount,
		FilterTypeID AS FilterTypeCount,
		Severity AS ServerityCount,
		COUNT(1) RulesCount,
		ISNULL(SUM(AlertsCount),0) AS AlertsCount,
		ISNULL(SUM(AlertsFlaggedCount),0) AS AlertsFlaggedCount,
		ISNULL(SUM(AlertsSentCount),0) AS AlertsSentCount,
		ISNULL(SUM(AlertsWinOSCount),0) AS AlertsWinOSCount,
		ISNULL(SUM(AlertsUnixOSCount),0) AS AlertsUnixOSCount,
		ISNULL(SUM(AlertsUKOSCount),0) AS AlertsUKOSCount
FROM	cteRulesStats
GROUP BY IsActive,FilterTypeID,Severity

