
SET NOCOUNT ON;

IF OBJECT_ID('tempdb..#ArchiveDBs') IS NOT NULL DROP TABLE #ArchiveDBs
IF OBJECT_ID('tempdb..#ArchiveVersions') IS NOT NULL DROP TABLE #ArchiveVersions

DECLARE @ArchivedDbName NVARCHAR(256),
		  @RowID INT = 1,
		  @Rows INT,
		  @SqlCmd NVARCHAR(MAX)

CREATE TABLE #ArchiveVersions
(
	DbName			NVARCHAR(256),
	ServerVersion	NVARCHAR(20),
	[create_date] [datetime],
	[compatibility_level] [tinyint],
	[collation_name] [sysname],
	[is_auto_close_on] [bit],
	[recovery_model_desc] [nvarchar](60),
	[page_verify_option_desc] [nvarchar](60),
	[is_auto_create_stats_on] [bit],
	[is_auto_update_stats_on] [bit],
	[is_auto_update_stats_async_on] [bit],
	[is_read_committed_snapshot_on] [bit]
);

SELECT	ROW_NUMBER() OVER (ORDER BY DbName) AS RowID,DbName
INTO	#ArchiveDBs
FROM	(
			SELECT	DbName 
			FROM	dbo.Archive_ArchiveDatabaseConfiguration
			UNION ALL
			SELECT 'ObserveIT_Archive_Template' AS DbName
		)d
WHERE	DbName IN (SELECT name COLLATE DATABASE_DEFAULT FROM sys.databases)
ORDER BY DbName

SET @Rows = @@ROWCOUNT

WHILE (@RowID <= @Rows)
BEGIN
	SELECT	@ArchivedDbName = DbName 
	FROM	#ArchiveDBs
	WHERE	RowID = @RowID

	SELECT	@SqlCmd = 
		   'INSERT INTO #ArchiveVersions
		   SELECT	TOP 1 ''' + @ArchivedDbName + ''' AS DbName,
					s.ServerVersion,
					d.create_date,
					d.compatibility_level,
					d.collation_name,
					d.is_auto_close_on,
					d.recovery_model_desc,
					d.page_verify_option_desc,
					d.is_auto_create_stats_on,
					d.is_auto_update_stats_on,
					d.is_auto_update_stats_async_on,
					d.is_read_committed_snapshot_on
				   
		   FROM	' + @ArchivedDbName + '.dbo.ServerVersions s
				INNER JOIN sys.databases d ON d.name = ''' + @ArchivedDbName + '''
		   ';
;	--PRINT(@SqlCmd)
	EXEC (@SqlCmd)
	
	SET @RowID = @RowID + 1
END

SELECT * FROM #ArchiveVersions ORDER BY DbName;
