/*
	Database Files Stats
*/

SET NOCOUNT ON;

DECLARE @DbName NVARCHAR(256),
		@Sql NVARCHAR(MAX) = ''

IF OBJECT_ID('tempdb..#DbFiles') IS NOT NULL DROP TABLE #DbFiles;

CREATE TABLE #DbFiles
(
	DbName nvarchar (128) NULL,
	LogicFileName sys.sysname NULL,
	PhysicalFileName nvarchar (260) NULL,
	FileID int NULL,
	FileGroupName sys.sysname NULL,
	CurrentSizeMB numeric (17, 6) NULL,
	SpaceUsedInMB numeric (17, 6) NULL,
	MaxSizeInMB numeric (17, 6) NULL,
	AutoGrowthType varchar (10) NULL,
	AutoGrowthInMB numeric (17, 6) NULL,
	FileTypeName nvarchar (60) NULL
)


DECLARE curDbs CURSOR FAST_FORWARD LOCAL READ_ONLY FOR
SELECT	name 
FROM	sys.databases 

OPEN curDbs 
FETCH NEXT FROM curDbs INTO @DbName

WHILE @@FETCH_STATUS = 0
BEGIN
	DECLARE @HasAccess INT = 0

	SET @Sql = N'
	BEGIN TRY
		SELECT TOP (1) @HasAccess = 1 FROM [' + @DbName + '].sys.all_objects;
	END TRY
	BEGIN CATCH
	END CATCH'

	EXEC sp_executesql @sql, N'@HasAccess INT OUT', @HasAccess = @HasAccess OUT;
	
	IF @HasAccess = 1
	BEGIN
		SET @Sql = N'
			USE [' + @DbName + '];

			INSERT INTO #DbFiles
			SELECT	DB_NAME() AS DbName,
					s.name AS LogicFileName, 
					s.physical_name AS PhysicalFileName,
					file_id AS FileID,
					CAST(CAST(g.name AS VARBINARY(256)) AS SYSNAME) AS [FileGroupName],
					size/128.0 AS CurrentSizeMB, 
					CAST(FILEPROPERTY(s.name, ''SpaceUsed'') AS INT)/128.0 AS SpaceUsedInMB,
					CASE WHEN s.[max_size] = -1 THEN -1 ELSE s.[max_size]/128.0 END AS [MaxSizeInMB],
					CASE WHEN s.is_percent_growth = 1 THEN ''Percentage'' ELSE ''MB'' END AS AutoGrowthType,
					CASE WHEN s.is_percent_growth = 1 THEN s.growth ELSE s.growth/128.0 END AS [AutoGrowthInMB],
					s.type_desc AS FileTypeName
			FROM	sys.database_files s
					LEFT JOIN sys.filegroups AS g ON ( ( s.type = 2 OR s.type = 0 ) AND ( s.drop_lsn IS NULL ) )  AND ( s.data_space_id = g.data_space_id )
		
		'
			--PRINT @Sql
		EXEC (@Sql)
	END

	FETCH NEXT FROM curDbs INTO @DbName
END

CLOSE curDbs   
DEALLOCATE curDbs

SELECT * FROM #DbFiles
ORDER BY 1,2


