SET NOCOUNT ON;
WITH ctePar AS (SELECT object_id, index_id, SUM(row_count) row_count, SUM(reserved_page_count) reserved_page_count 
	FROM sys.dm_db_partition_stats P GROUP BY object_id, index_id)

SELECT  OBJECT_NAME(all_i.object_id) AS [Table],
        CASE WHEN all_i.name IS NULL THEN 'HEAP'
             ELSE all_i.name
        END AS [Index],
        CASE WHEN all_i.index_id IS NULL THEN 'HEAP'
             WHEN all_i.index_id = 1 THEN 'CLUSTERED'
             ELSE 'NONCLUSTERED'
        END AS [Index Type],
        user_lookups + user_scans + user_seeks AS [Index Usage],
        user_updates AS [Index Updates],
        STUFF(( SELECT  ',' + c.name
                FROM    sys.index_columns ic3
                        JOIN sys.columns c ON c.object_id = ic3.object_id
                                              AND c.column_id = ic3.column_id
                WHERE   ic3.object_id = all_iu.object_id
                        AND ic3.index_id = all_iu.index_id
                ORDER BY ic3.key_ordinal
              FOR
                XML PATH('')
              ),1,1,'') AS [Index Columns],
	    ps.reserved_page_count*8.0/1024 as Size_MB,
		ps.row_count as Rows,
        ( SELECT    MAX(create_date)
          FROM      sys.databases
          WHERE     name IN ('ObserveIT','ObserveIT_Data')
        ) AS [Reset Date],
        GETDATE() AS [Report Date]
FROM    sys.indexes all_i
        INNER JOIN sys.dm_db_index_usage_stats all_iu ON all_iu.object_id = all_i.object_id AND all_iu.index_id = all_i.index_id
		INNER JOIN ctePar ps ON ps.object_id = all_iu.object_id AND ps.index_id = all_iu.index_id
WHERE   EXISTS (SELECT i.object_id
                 FROM   sys.dm_db_index_usage_stats iu
                        JOIN sys.indexes i ON iu.object_id = i.object_id
                                              AND iu.index_id = i.index_id
                                              AND all_i.object_id = i.object_id
                 WHERE  user_lookups + user_scans + user_seeks = 0 AND row_count > 1000)
ORDER BY [Table], [Index Usage]
