SET NOCOUNT ON;

;WITH cteLog AS
(
	SELECT	CreatedDate,
			ROW_NUMBER() OVER (ORDER BY CreatedDate) AS LogOrderId
	FROM	dbo.AuditLog l
),cteLogIntervals AS
(
	SELECT	CASE WHEN DATEDIFF(mi,l1.CreatedDate,l2.CreatedDate) = 0 THEN 1 ELSE DATEDIFF(mi,l1.CreatedDate,l2.CreatedDate) END AS  LoginsIntervalsInMinutes
	FROM	cteLog l1
			INNER JOIN cteLog l2 ON l1.LogOrderId + 1 = l2.LogOrderId
),cteLogIntervalsAggr AS
(
	SELECT	--MIN(LoginsIntervalsInMinutes) AS  MinLoginsIntervalsInMinutes,
			MAX(LoginsIntervalsInMinutes) AS  MaxLoginsIntervalsInMinutes,
			AVG(LoginsIntervalsInMinutes) AS  AvgLoginsIntervalsInMinutes,
			STDEVP(LoginsIntervalsInMinutes) AS StdLoginsIntervalsInMinutes
	FROM	cteLogIntervals
),cteLogins AS
(
	SELECT  COUNT(1) AS TotalLogins,
			COUNT(DISTINCT l.DomainName + l.Username) AS TotalUniqueUsers,
			COUNT(DISTINCT l.IPAddress) AS TotalUniqueIPs,
			MAX(ISNULL(s.SessionsInLogin,0))  AS MaxSessionsInLogins,
			AVG(ISNULL(s.SessionsInLogin,0) * 1.00)  AS AvgSessionsInLogins,
			MAX(ISNULL(s.UniqueSessionsInLogin,0))  AS MaxUniqueSessionsInLogins,
			AVG(ISNULL(s.UniqueSessionsInLogin,0)* 1.00)  AS AvgUniqueSessionsInLogins
	FROM    dbo.AuditLog l WITH (NOLOCK)
			OUTER APPLY (SELECT COUNT(1) AS SessionsInLogin,COUNT(DISTINCT SessionID) AS UniqueSessionsInLogin FROM dbo.AuditLogSessions s WHERE l.AuditLogId = s.AuditLogId) s
),cteLoginsSessions AS
(
	SELECT  COUNT(1) AS TotalSessions,
			COUNT(DISTINCT ALS.SessionID) AS TotalUniqueSessions
	FROM    dbo.AuditLogSessions ALS 
)

SELECT	l.*,a.*,s.*
FROM	cteLogIntervalsAggr a CROSS JOIN cteLogins l CROSS JOIN cteLoginsSessions s
