SET NOCOUNT ON;

If OBJECT_ID('tempdb..#OSPerfCounters') IS NULL
BEGIN
	CREATE TABLE #OSPerfCounters(
		DateAdded datetime NOT NULL
		, Interval_In_Seconds int NOT NULL
		, Batch_Requests_Sec int NULL
		, Cache_Hit_Ratio float NULL
		, Free_Pages int NULL
		, Lazy_Writes_Sec int NULL
		, Memory_Grants_Pending int NULL
		, Deadlocks int NULL
		, Page_Life_Exp int NULL
		, Page_Lookups_Sec int NULL
		, Page_Reads_Sec int NULL
		, Page_Writes_Sec int NULL
		, SQL_Compilations_Sec int NULL
		, SQL_Recompilations_Sec int NULL
		, ServerMemoryTarget_KB int NULL
		, ServerMemoryTotal_KB int NULL
		, Transactions_Sec int NULL
	)

	CREATE UNIQUE CLUSTERED INDEX IX_#OSPerfCounters_DateAdded_U_C ON #OSPerfCounters
	(
		DateAdded
	) WITH (FillFactor = 100)
END

DECLARE @FirstCollectionTime DateTime
    , @SecondCollectionTime DateTime
    , @NumberOfSeconds Int
    , @BatchRequests Float
    , @LazyWrites Float
    , @Deadlocks BigInt
    , @PageLookups Float
    , @PageReads Float
    , @PageWrites Float
    , @SQLCompilations Float
    , @SQLRecompilations Float
    , @Transactions Float

DECLARE @CounterPrefix NVARCHAR(30)
SET @CounterPrefix = CASE WHEN @@SERVICENAME = 'MSSQLSERVER'
                            THEN 'SQLServer:'
                        ELSE 'MSSQL$' + @@SERVICENAME + ':'
                        END


IF OBJECT_ID('tempdb..#FirstCollection') IS NOT NULL 
BEGIN
    DROP TABLE #FirstCollection
END

SELECT @FirstCollectionTime = GETDATE()

SELECT counter_name, cntr_value--, cntr_type --I considered dynamically doing each counter type, but decided manual was better in this case
INTO #FirstCollection 
FROM sys.dm_os_performance_counters 
WHERE object_name like @CounterPrefix + '%'
    AND instance_name IN ('', '_Total')
    AND counter_name IN ( N'Batch Requests/sec'
                        , N'Buffer cache hit ratio'
                        , N'Buffer cache hit ratio base'
                        , N'Free Pages'
                        , N'Lazy Writes/sec'
                        , N'Memory Grants Pending'
                        , N'Number of Deadlocks/sec'
                        , N'Page life expectancy'
                        , N'Page Lookups/Sec'
                        , N'Page Reads/Sec'
                        , N'Page Writes/Sec'
                        , N'SQL Compilations/sec'
                        , N'SQL Re-Compilations/sec'
                        , N'Target Server Memory (KB)'
                        , N'Total Server Memory (KB)'
                        , N'Transactions/sec')

WAITFOR DELAY '00:00:10'

IF OBJECT_ID('tempdb..#SecondCollection') IS NOT NULL
BEGIN
    DROP TABLE #SecondCollection
END


SELECT @SecondCollectionTime = GETDATE()

--Grab the current values from dm_os_performance_counters
--Doesn't do anything by instance or database because this is good enough and works unaltered in all envirornments
SELECT counter_name, cntr_value--, cntr_type --I considered dynamically doing each counter type, but decided manual was better in this case
INTO #SecondCollection 
FROM sys.dm_os_performance_counters 
WHERE object_name like @CounterPrefix + '%'
    AND instance_name IN ('', '_Total')
    AND counter_name IN ( N'Batch Requests/sec'
                        , N'Buffer cache hit ratio'
                        , N'Buffer cache hit ratio base'
                        , N'Free Pages'
                        , N'Lazy Writes/sec'
                        , N'Memory Grants Pending'
                        , N'Number of Deadlocks/sec'
                        , N'Page life expectancy'
                        , N'Page Lookups/Sec'
                        , N'Page Reads/Sec'
                        , N'Page Writes/Sec'
                        , N'SQL Compilations/sec'
                        , N'SQL Re-Compilations/sec'
                        , N'Target Server Memory (KB)'
                        , N'Total Server Memory (KB)'
                        , N'Transactions/sec')

SELECT @NumberOfSeconds = DATEDIFF(ss, @FirstCollectionTime, @SecondCollectionTime)


--I put these in alphabetical order by counter_name, not column name.  It looks a bit odd, but makes sense to me
--Deadlocks are odd here.  I keep track of the number of deadlocks in the time period, not average number of deadlocks per second.
  --AKA, I keep track of things the way I would refer to them when I talk to someone.  "We had 2 deadlocks in the last 5 minutes", not "We averaged .00002 deadlocks per second there"
INSERT INTO #OSPerfCounters (DateAdded, Interval_In_Seconds, Batch_Requests_Sec, Cache_Hit_Ratio, Free_Pages, Lazy_Writes_Sec, Memory_Grants_Pending
    , Deadlocks, Page_Life_Exp, Page_Lookups_Sec, Page_Reads_Sec, Page_Writes_Sec, SQL_Compilations_Sec, SQL_Recompilations_Sec
    , ServerMemoryTarget_KB, ServerMemoryTotal_KB, Transactions_Sec)
SELECT @SecondCollectionTime
	, @NumberOfSeconds
    , Batch_Request_Sec        = ((SELECT cntr_value FROM #SecondCollection WHERE counter_name = N'Batch Requests/sec') - (SELECT cntr_value FROM #FirstCollection WHERE counter_name = N'Batch Requests/sec')) / @NumberOfSeconds
    , Cache_Hit_Ratio        = (SELECT cntr_value FROM #SecondCollection WHERE counter_name = N'Buffer cache hit ratio')/(SELECT cntr_value FROM #SecondCollection WHERE counter_name = N'Buffer cache hit ratio base')
    , Free_Pages            = (SELECT cntr_value FROM #SecondCollection WHERE counter_name =N'Free pages')
    , Lazy_Writes_Sec        = ((SELECT cntr_value FROM #SecondCollection WHERE counter_name = N'Lazy Writes/sec') - (SELECT cntr_value FROM #FirstCollection WHERE counter_name = N'Lazy Writes/sec')) / @NumberOfSeconds
    , Memory_Grants_Pending    = (SELECT cntr_value FROM #SecondCollection WHERE counter_name = N'Memory Grants Pending')
    , Deadlocks                = ((SELECT cntr_value FROM #SecondCollection WHERE counter_name = N'Number of Deadlocks/sec') - (SELECT cntr_value FROM #FirstCollection WHERE counter_name = N'Number of Deadlocks/sec')) 
    , Page_Life_Exp         = (SELECT cntr_value FROM #SecondCollection WHERE counter_name = N'Page life expectancy')
    , Page_Lookups_Sec      = ((SELECT cntr_value FROM #SecondCollection WHERE counter_name = N'Page lookups/sec') - (SELECT cntr_value FROM #FirstCollection WHERE counter_name = N'Page lookups/sec')) / @NumberOfSeconds
    , Page_Reads_Sec        = ((SELECT cntr_value FROM #SecondCollection WHERE counter_name = N'Page reads/sec') - (SELECT cntr_value FROM #FirstCollection WHERE counter_name = N'Page reads/sec')) / @NumberOfSeconds
    , Page_Writes_Sec        = ((SELECT cntr_value FROM #SecondCollection WHERE counter_name = N'Page writes/sec') - (SELECT cntr_value FROM #FirstCollection WHERE counter_name = N'Page writes/sec')) / @NumberOfSeconds
    , SQL_Compilations_Sec  = ((SELECT cntr_value FROM #SecondCollection WHERE counter_name = N'SQL Compilations/sec') - (SELECT cntr_value FROM #FirstCollection WHERE counter_name = N'SQL Compilations/sec')) / @NumberOfSeconds
    , SQL_Recompilations_Sec= ((SELECT cntr_value FROM #SecondCollection WHERE counter_name = N'SQL Re-Compilations/sec') - (SELECT cntr_value FROM #FirstCollection WHERE counter_name = N'SQL Re-Compilations/sec')) / @NumberOfSeconds
    , ServerMemoryTarget_KB = (SELECT cntr_value FROM #SecondCollection WHERE counter_name = N'Target Server Memory (KB)')
    , ServerMemoryTotal_KB  = (SELECT cntr_value FROM #SecondCollection WHERE counter_name = N'Total Server Memory (KB)')
    , Transactions_Sec        = ((SELECT cntr_value FROM #SecondCollection WHERE counter_name = N'Transactions/sec') - (SELECT cntr_value FROM #FirstCollection WHERE counter_name = N'Transactions/sec')) / @NumberOfSeconds



SELECT	TOP 1000 * 
FROM	#OSPerfCounters
ORDER BY DateAdded
