SET NOCOUNT ON;

IF OBJECT_ID('tempdb..#ignorable_waits') IS NOT NULL DROP TABLE #ignorable_waits;

create table #ignorable_waits (wait_type nvarchar(256) PRIMARY KEY);

insert #ignorable_waits (wait_type) VALUES ('REQUEST_FOR_DEADLOCK_SEARCH');
insert #ignorable_waits (wait_type) VALUES ('SQLTRACE_INCREMENTAL_FLUSH_SLEEP');
insert #ignorable_waits (wait_type) VALUES ('SQLTRACE_BUFFER_FLUSH');
insert #ignorable_waits (wait_type) VALUES ('LAZYWRITER_SLEEP');
insert #ignorable_waits (wait_type) VALUES ('XE_TIMER_EVENT');
insert #ignorable_waits (wait_type) VALUES ('XE_DISPATCHER_WAIT');
insert #ignorable_waits (wait_type) VALUES ('FT_IFTS_SCHEDULER_IDLE_WAIT');
insert #ignorable_waits (wait_type) VALUES ('LOGMGR_QUEUE');
insert #ignorable_waits (wait_type) VALUES ('CHECKPOINT_QUEUE');
insert #ignorable_waits (wait_type) VALUES ('BROKER_TO_FLUSH');
insert #ignorable_waits (wait_type) VALUES ('BROKER_TASK_STOP');
insert #ignorable_waits (wait_type) VALUES ('BROKER_EVENTHANDLER');
insert #ignorable_waits (wait_type) VALUES ('SLEEP_TASK');
insert #ignorable_waits (wait_type) VALUES ('WAITFOR');
insert #ignorable_waits (wait_type) VALUES ('DBMIRROR_DBM_MUTEX')
insert #ignorable_waits (wait_type) VALUES ('DBMIRROR_EVENTS_QUEUE')
insert #ignorable_waits (wait_type) VALUES ('DBMIRRORING_CMD');
insert #ignorable_waits (wait_type) VALUES ('DISPATCHER_QUEUE_SEMAPHORE');
insert #ignorable_waits (wait_type) VALUES ('BROKER_RECEIVE_WAITFOR');
insert #ignorable_waits (wait_type) VALUES ('CLR_AUTO_EVENT');
insert #ignorable_waits (wait_type) VALUES ('DIRTY_PAGE_POLL');
insert #ignorable_waits (wait_type) VALUES ('HADR_FILESTREAM_IOMGR_IOCOMPLETION');
insert #ignorable_waits (wait_type) VALUES ('ONDEMAND_TASK_QUEUE');
insert #ignorable_waits (wait_type) VALUES ('FT_IFTSHC_MUTEX');
insert #ignorable_waits (wait_type) VALUES ('CLR_MANUAL_EVENT');
insert #ignorable_waits (wait_type) VALUES ('SP_SERVER_DIAGNOSTICS_SLEEP');
insert #ignorable_waits (wait_type) VALUES ('QDS_CLEANUP_STALE_QUERIES_TASK_MAIN_LOOP_SLEEP');
insert #ignorable_waits (wait_type) VALUES ('QDS_PERSIST_TASK_MAIN_LOOP_SLEEP');


IF OBJECT_ID('tempdb..#results') IS NOT NULL DROP TABLE #results;

;WITH cteWaits AS
(
	SELECT	ROW_NUMBER() OVER(ORDER BY wait_time_ms DESC) AS rn,
			wait_type,
			wait_time_ms /1000. AS wait_time_s,
			SUM(wait_time_ms) OVER () /1000. AS wait_time_s_total
	FROM	sys.dm_os_wait_stats
	WHERE	wait_type NOT LIKE'%SLEEP%'
			and wait_type not in(SELECT wait_type FROM #ignorable_waits)
),cteWaitsSubTotal AS
(
	SELECT	*,(SELECT ISNULL(SUM(w2.wait_time_s),0) FROM cteWaits w2 WHERE w2.rn < w1.rn) + w1.wait_time_s AS  wait_time_s_sub_total
	FROM	cteWaits w1
)
SELECT	wait_type, 
		CAST(wait_time_s AS DECIMAL(12,2)) AS wait_time_s,
		CAST(wait_time_s/wait_time_s_total AS DECIMAL(12,2)) AS pct,
		CAST(wait_time_s_sub_total/wait_time_s_total AS DECIMAL(12,2)) AS running_pct,
		getdate()as running_date,
		'ServiceStart' AS RunningPeriod
INTO	#results
FROM	cteWaitsSubTotal
WHERE	wait_time_s_total > 0 AND CAST(wait_time_s_sub_total/wait_time_s_total AS DECIMAL(12,2))  <= 0.99


--DBCC SQLPERF("sys.dm_os_wait_stats",CLEAR);
--WAITFOR DELAY '00:00:15';

--;WITH cteWaits AS
--(
--	SELECT	ROW_NUMBER() OVER(ORDER BY wait_time_ms DESC) AS rn,
--			wait_type,
--			wait_time_ms /1000. AS wait_time_s,
--			SUM(wait_time_ms) OVER () /1000. AS wait_time_s_total
--	FROM	sys.dm_os_wait_stats
--	WHERE	wait_type NOT LIKE'%SLEEP%'
--			and wait_type not in(SELECT wait_type FROM #ignorable_waits)
--),cteWaitsSubTotal AS
--(
--	SELECT	*,(SELECT ISNULL(SUM(w2.wait_time_s),0) FROM cteWaits w2 WHERE w2.rn < w1.rn) + w1.wait_time_s AS  wait_time_s_sub_total
--	FROM	cteWaits w1
--)
--INSERT INTO	#results
--SELECT	wait_type, 
--		CAST(wait_time_s AS DECIMAL(12,2)) AS wait_time_s,
--		CAST(wait_time_s/wait_time_s_total AS DECIMAL(12,2)) AS pct,
--		CAST(wait_time_s_sub_total/wait_time_s_total AS DECIMAL(12,2)) AS running_pct,
--		getdate()as running_date,
--		'15Seconds' AS RunningPeriod
--FROM	cteWaitsSubTotal
--WHERE	wait_time_s_total > 0 AND CAST(wait_time_s_sub_total/wait_time_s_total AS DECIMAL(12,2))  <= 0.99

SELECT	* 
FROM	#results 
ORDER BY RunningPeriod DESC ,wait_time_s desc;
