/*
	Users Daily Activity
*/

SET NOCOUNT ON;

DECLARE @ServerOffset INT = DATEDIFF(mi, SYSUTCDATETIME(), SYSDATETIME())


--Get working hours
;WITH cteSessions AS
(
	SELECT	s.DomainName + '\' + s.LoginName + 
			CASE WHEN s.UserAuthentication <> 'n/a' 
				THEN '-' + ISNULL(s.UserAuthentication,'') + '\' + ISNULL(s.UserName,'') 
				ELSE ''
			END AS UserFullName,
			DATEADD(mi,UTCOffsetMins-@ServerOffset,s.SessionDate)  AS SessionStartTime,
			DATEADD(mi,UTCOffsetMins-@ServerOffset,s.SessionLastActivityDate) AS SessionEndTime,
			CASE si.OperatingSystemType WHEN 1 THEN 'Win' ELSE 'Unix' END AS OSName,
			s.*
	FROM	dbo.Session s
			INNER JOIN dbo.ServerInventory si ON si.SrvID = s.SrvID
	--WHERE	s.ExtSlides > 0 --s.SessionDay > DATEADD(dd,-30,GETDATE())
),cteUserStatsDaily AS
(
	SELECT	UserFullName,
			CAST(SessionStartTime AS DATE) AS SessionUserDay,
			OSName,
			DATEDIFF(mi,MIN(SessionStartTime),MAX(SessionEndTime)) / 60.0 AS WorkingHours
	FROM cteSessions
	--WHERE	DATEDIFF(mi,SessionStartTime,SessionEndTime) / 60.0 BETWEEN 2 AND 23
	GROUP BY UserFullName,CAST(SessionStartTime AS DATE),OSName
),cteDailyWorkingHours AS
(
	SELECT	SessionUserDay,OSName,AVG(WorkingHours) AS WorkingHours
	FROM	cteUserStatsDaily
	--WHERE	WorkingHours < 24
	GROUP BY SessionUserDay,OSName
)
SELECT	s.SessionDay,
		S.OSName, 
		COUNT(1) AS SessionsCount,
		SUM(s.ExtSlides) AS SlidesCount,
		COUNT(DISTINCT UserFullName) AS UsersCount,
		COUNT(DISTINCT s.SrvID) AS EndpointsCount,
		COUNT(CASE WHEN NULLIF(s.ClientName,'') IS NOT NULL THEN 1 ELSE NULL END) AS RemoteSessionsCount,
		COUNT(CASE WHEN s.ExtSlides > 0 THEN 1 ELSE NULL END) AS SessionsWithSlides,
		MIN(SessionDate) AS FirstSessionTime,
		MAX(SessionLastActivityDate) AS LastSessionTime,
		COUNT(1)/COUNT(DISTINCT UserFullName) AS SessionsPerUser,
		SUM(ExtSlides) /COUNT(DISTINCT UserFullName) AS SlidesPerUser,
		SUM(ExtSlides) /COUNT(1) AS SlidesPerSession,
		wh.WorkingHours,
		SUM(s.AlertCount) AS Alerts
FROM	cteSessions s WITH (NOLOCK)
		LEFT JOIN cteDailyWorkingHours wh ON wh.SessionUserDay = s.SessionDay AND wh.OSName = s.OSName
GROUP BY s.SessionDay,S.OSName,wh.WorkingHours
ORDER BY 1,2
