/*
	Tables statistics
*/

SET NOCOUNT ON;

IF OBJECT_ID('tempdb..#TablesStats') IS NOT NULL DROP TABLE #TablesStats;

SELECT  DbName = CAST('ObserveIT' AS VARCHAR(256)),
		TableName = a2.name ,
		SchemaName = a3.name ,
		RowCounts = a1.rows ,
		TotalSpaceMB = ( a1.reserved + ISNULL(a4.reserved, 0) ) / 128 ,
		UsedSpaceMB = a1.used / 128,
		UnusedSpaceMB = ( CASE WHEN ( a1.reserved + ISNULL(a4.reserved, 0) ) > a1.used THEN ( a1.reserved + ISNULL(a4.reserved, 0) ) - a1.used ELSE 0 END ) / 128
INTO	#TablesStats
FROM    ( SELECT    ps.object_id ,
					[rows] = SUM(CASE WHEN ( ps.index_id < 2 ) THEN row_count ELSE 0 END) ,
					reserved = SUM(ps.reserved_page_count) ,
					data = SUM(CASE WHEN ( ps.index_id < 2 )
									THEN ( ps.in_row_data_page_count + ps.lob_used_page_count + ps.row_overflow_used_page_count )
									ELSE ( ps.lob_used_page_count + ps.row_overflow_used_page_count )
								END) ,
					used = SUM(ps.used_page_count)
			FROM      ObserveIT.sys.dm_db_partition_stats ps
			GROUP BY  ps.object_id
		) AS a1
		INNER JOIN ObserveIT.sys.all_objects a2 ON a1.object_id = a2.object_id
		INNER JOIN ObserveIT.sys.schemas a3 ON a2.schema_id = a3.schema_id
		LEFT JOIN ( SELECT  it.parent_id ,
							reserved = SUM(ps.reserved_page_count) ,
							used = SUM(ps.used_page_count)
					FROM    ObserveIT.sys.dm_db_partition_stats ps
							INNER JOIN ObserveIT.sys.internal_tables it ON it.object_id = ps.object_id
					WHERE   it.internal_type IN ( 202, 204 )
					GROUP BY it.parent_id
					) AS a4 ON a4.parent_id = a1.object_id
WHERE   a2.type <> 'S' AND a2.type <> 'IT'


INSERT INTO #TablesStats
SELECT  DbName = 'ObserveIT_Data' ,
        TableName = a2.name ,
        SchemaName = a3.name ,
        RowCounts = a1.rows ,
        TotalSpaceMB = ( a1.reserved + ISNULL(a4.reserved, 0) ) / 128 ,
        UsedSpaceMB = ( a1.data + ( CASE	WHEN ( a1.used + ISNULL(a4.used, 0) ) > a1.data THEN ( a1.used + ISNULL(a4.used, 0) ) - a1.data ELSE 0 END ) ) / 128 ,
        UnusedSpaceMB = ( CASE WHEN ( a1.reserved + ISNULL(a4.reserved, 0) ) > a1.used THEN ( a1.reserved + ISNULL(a4.reserved, 0) ) - a1.used ELSE 0 END ) / 128
FROM    ( SELECT    ps.object_id ,
                    [rows] = SUM(CASE WHEN ( ps.index_id < 2 ) THEN row_count ELSE 0 END) ,
                    reserved = SUM(ps.reserved_page_count) ,
                    data = SUM(CASE WHEN ( ps.index_id < 2 )
                                    THEN ( ps.in_row_data_page_count + ps.lob_used_page_count + ps.row_overflow_used_page_count )
                                    ELSE ( ps.lob_used_page_count + ps.row_overflow_used_page_count )
                               END) ,
                    used = SUM(ps.used_page_count)
          FROM      ObserveIT_Data.sys.dm_db_partition_stats ps
          GROUP BY  ps.object_id
        ) AS a1
        INNER JOIN ObserveIT_Data.sys.all_objects a2 ON a1.object_id = a2.object_id
        INNER JOIN ObserveIT_Data.sys.schemas a3 ON a2.schema_id = a3.schema_id
        LEFT JOIN ( SELECT  it.parent_id ,
                            reserved = SUM(ps.reserved_page_count) ,
                            used = SUM(ps.used_page_count)
                    FROM    ObserveIT_Data.sys.dm_db_partition_stats ps
                            INNER JOIN ObserveIT_Data.sys.internal_tables it ON it.object_id = ps.object_id
                    WHERE   it.internal_type IN ( 202, 204 )
                    GROUP BY it.parent_id
                  ) AS a4 ON a4.parent_id = a1.object_id
WHERE   a2.type <> 'S' AND a2.type <> 'IT'
ORDER BY DbName, TableName


IF EXISTS (SELECT * FROM sys.databases WHERE name = 'ObserveIT_Analytics')
BEGIN
	INSERT INTO #TablesStats
	SELECT  DbName = 'ObserveIT_Analytics' ,
			TableName = a2.name ,
			SchemaName = a3.name ,
			RowCounts = a1.rows ,
			TotalSpaceMB = ( a1.reserved + ISNULL(a4.reserved, 0) ) / 128 ,
			UsedSpaceMB = ( a1.data + ( CASE	WHEN ( a1.used + ISNULL(a4.used, 0) ) > a1.data THEN ( a1.used + ISNULL(a4.used, 0) ) - a1.data ELSE 0 END ) ) / 128 ,
			UnusedSpaceMB = ( CASE WHEN ( a1.reserved + ISNULL(a4.reserved, 0) ) > a1.used THEN ( a1.reserved + ISNULL(a4.reserved, 0) ) - a1.used ELSE 0 END ) / 128
	FROM    ( SELECT    ps.object_id ,
						[rows] = SUM(CASE WHEN ( ps.index_id < 2 ) THEN row_count ELSE 0 END) ,
						reserved = SUM(ps.reserved_page_count) ,
						data = SUM(CASE WHEN ( ps.index_id < 2 )
										THEN ( ps.in_row_data_page_count + ps.lob_used_page_count + ps.row_overflow_used_page_count )
										ELSE ( ps.lob_used_page_count + ps.row_overflow_used_page_count )
									END) ,
						used = SUM(ps.used_page_count)
				FROM      ObserveIT_Analytics.sys.dm_db_partition_stats ps
				GROUP BY  ps.object_id
			) AS a1
			INNER JOIN ObserveIT_Analytics.sys.all_objects a2 ON a1.object_id = a2.object_id
			INNER JOIN ObserveIT_Analytics.sys.schemas a3 ON a2.schema_id = a3.schema_id
			LEFT JOIN ( SELECT  it.parent_id ,
								reserved = SUM(ps.reserved_page_count) ,
								used = SUM(ps.used_page_count)
						FROM    ObserveIT_Analytics.sys.dm_db_partition_stats ps
								INNER JOIN ObserveIT_Analytics.sys.internal_tables it ON it.object_id = ps.object_id
						WHERE   it.internal_type IN ( 202, 204 )
						GROUP BY it.parent_id
						) AS a4 ON a4.parent_id = a1.object_id
	WHERE   a2.type <> 'S' AND a2.type <> 'IT'
	ORDER BY DbName, TableName
END


select	DbName,
		TableName,
		SchemaName,
		RowCounts,
		UsedSpaceMB,
		UnusedSpaceMB,
		TotalSpaceMB
FROM	#TablesStats
--WHERE	RowCounts >1000 
ORDER by DbName,TableName

