--USE ObserveIT
--GO

--parameters
DECLARE @FromDay DATE, -- optional (previous day by default), format: YYYY-MM-DD 
        @ToDay DATE,   -- optional (previous day by default), format: YYYY-MM-DD 
        @LayoutOption VARCHAR(20) = 'detailed', -- options: detailed | summary
        @DefaultDayOption VARCHAR(20) = 'most-active' -- Optional: yesturday | most-active 
        ;

SET NOCOUNT ON; 
SET ANSI_WARNINGS OFF;

IF OBJECT_ID('tempdb..#events') IS NOT NULL DROP TABLE #events;

CREATE TABLE #events
(
    EventKind varchar(20),
    Day DATE,
    Minute  DATETIME2(0),
    Endpoints INT,
    Sessions INT,
    Events  INT,
    EventsPerSecond DECIMAL(18,2)
)


DECLARE @Day DATE, @Message NVARCHAR(MAX) = '';

IF @FromDay IS NULL
    IF @DefaultDayOption = 'most-active'
        SET @FromDay = (
            SELECT TOP 1 CAST(s.SessionDate AS DATE)
            FROM    dbo.Session s  WITH (NOLOCK)
            WHERE   s.SessionDate > DATEADD(dd, -{0}, GETDATE())
            GROUP BY CAST(s.SessionDate AS DATE) 
            ORDER BY SUM(s.ExtSlides) DESC
        );
    ELSE 
        SET @FromDay = CAST(DATEADD(dd,-1,GETDATE()) AS DATE);
 
IF @ToDay IS NULL
    SET @ToDay = @FromDay;

SET @Day = @FromDay;

WHILE @Day <= @ToDay
BEGIN 
    
    SET @Message = '[' + CONVERT(VARCHAR(20), GETDATE(), 120) + '] ' + 'Processing day: ' + CONVERT(VARCHAR(10), @Day, 120);
    RAISERROR(@Message, 10, 1) WITH NOWAIT;

    IF OBJECT_ID('tempdb..#sessions') IS NOT NULL DROP TABLE #sessions;
	IF OBJECT_ID('tempdb..#tmpUI') IS NOT NULL DROP TABLE #tmpUI;

    ;WITH sessionDetails AS
    (
        SELECT  CASE WHEN si.OperatingSystemType = 1 THEN 'ui-daily' ELSE 'cmd-daily' END AS EventKind,
                s.SessionDay AS Day, 
                s.SessionDay AS Minute,
                s.SrvID,
                s.ExtSlides
        FROM    Session s 
                INNER JOIN ServerInventory si ON s.SrvID = si.SrvID
        WHERE   s.SessionDay = @day 
    
    )
    SELECT  EventKind,
            Day, 
            Minute,
            COUNT(DISTINCT SrvID) AS Endpoints,
            COUNT(1) AS Sessios
    INTO    #sessions
    FROM    sessionDetails 
    GROUP BY EventKind, Day, Minute
    OPTION (RECOMPILE);

    --UI
    SELECT  SessionDay AS Day, 
            DATEADD(mi, DATEDIFF(mi, 0, SDateTime), 0) AS Minute,
            COUNT(DISTINCT SrvID) AS Endpoints,
            COUNT(DISTINCT SSessionID) AS Sessions,
            COUNT(1) AS Events, 
            COUNT(DISTINCT CASE WHEN SBytes > 0 THEN SrvID ELSE NULL END) AS Endpoints_Images,
            COUNT(DISTINCT CASE WHEN SBytes > 0 THEN SSessionID ELSE NULL END) AS Sessions_Images,
            SUM(CASE WHEN SBytes > 0 THEN 1 ELSE 0 END) AS Events_Images
    INTO    #tmpUI
    FROM    dbo.Screenshot (NOLOCK)
    WHERE   SessionDay = @day AND SDateTime >= @day AND SDateTime < DATEADD(dd, 1, @day)
    GROUP BY SessionDay, DATEADD(mi, DATEDIFF(mi, 0, SDateTime), 0)
    OPTION (RECOMPILE);

    --ui metadata
    INSERT INTO #events(EventKind, Day, Minute, Endpoints, Sessions, Events, EventsPerSecond)
    SELECT  CAST('ui' AS VARCHAR(20)) AS EventKind, Day, Minute, Endpoints, Sessions, Events, CAST(Events / 60.00 AS decimal(18,2))  AS EventsPerSecond
    --INTO  #events
    FROM    #tmpUI

    --ui images
    INSERT INTO #events(EventKind, Day, Minute, Endpoints, Sessions, Events, EventsPerSecond)
    SELECT  CAST('ui-images' AS VARCHAR(20)) AS EventKind, Day, Minute, Endpoints_Images, Sessions_Images, Events_Images, 
            CAST(Events_Images / 60.00 AS decimal(18,2))  AS EventsPerSecond
    FROM    #tmpUI

    --FAM
    INSERT INTO #events(EventKind, Day, Minute, Endpoints, Sessions, Events, EventsPerSecond)
    SELECT  'fam' AS EventKind,
            ActionDay AS Day, 
            DATEADD(mi, DATEDIFF(mi, 0, DATEADD(mi, UTCOffsetMins, ActionUTCTime)), 0) AS Minute,
            --DATEADD(mi, DATEDIFF(mi, 0, ActionUTCTime), 0) AS Minute,
            COUNT(DISTINCT SrvID) AS Endpoints,
            COUNT(DISTINCT SessionID) AS Sessions,
            COUNT(1) AS Events, 
            COUNT(1) / 60.0 AS EventsPerSecond
    FROM    dbo.FileMonitoringAction (NOLOCK)
    WHERE   ActionDay >= DATEADD(dd,-1, @day) AND ActionDay < DATEADD(dd,2, @day) -- actionday partition in UTC
            AND  DATEADD(mi, UTCOffsetMins, ActionUTCTime) >= @day AND  DATEADD(mi, UTCOffsetMins, ActionUTCTime) < DATEADD(dd, 1, @day)
    GROUP BY ActionDay, DATEADD(mi, DATEDIFF(mi, 0, DATEADD(mi, UTCOffsetMins, ActionUTCTime)), 0) --DATEADD(mi, DATEDIFF(mi, 0, ActionUTCTime), 0)
    OPTION (RECOMPILE);

    --CMD
    INSERT INTO #events(EventKind, Day, Minute, Endpoints, Sessions, Events, EventsPerSecond)
    SELECT  'cmd' AS EventKind,
            uc.SessionDay AS Day, 
            DATEADD(mi, DATEDIFF(mi, 0, uc.CommandDate), 0) AS Minute,
            COUNT(DISTINCT s.SrvID) AS Endpoints,
            COUNT(DISTINCT uc.SessionID) AS Sessions,
            COUNT(1) AS Events, 
            COUNT(1) / 60.0 AS EventsPerSecond
    FROM    dbo.UnixCommand uc (NOLOCK)
            INNER JOIN Session s (NOLOCK) ON uc.SessionId = s.SessionID
    WHERE   uc.SessionDay = @day  AND uc.CommandDate >= @day AND uc.CommandDate < DATEADD(dd, 1, @day)
    GROUP BY uc.SessionDay, DATEADD(mi, DATEDIFF(mi, 0, uc.CommandDate), 0)
    OPTION (RECOMPILE);

    
    SET @Day = DATEADD(dd, 1, @Day);
END


SET @Message = '[' + CONVERT(VARCHAR(20), GETDATE(), 120) + '] ' + 'done';
RAISERROR(@Message, 10, 1) WITH NOWAIT;

;WITH events AS
(
    SELECT 'Client' AS Type, 
        Minute AS DateMinute,
        
        SUM(CASE WHEN EventKind = 'ui' THEN Events ELSE 0 END) AS Slides, 
        SUM(CASE WHEN EventKind = 'ui' THEN Sessions ELSE 0 END) AS Sessions, 
        SUM(CASE WHEN EventKind = 'ui' THEN Endpoints ELSE 0 END) AS Endpoints, 

        SUM(CASE WHEN EventKind = 'ui' THEN Events ELSE 0 END) AS UI_Events, 
        SUM(CASE WHEN EventKind = 'ui' THEN Sessions ELSE 0 END) AS UI_Sessions, 
        SUM(CASE WHEN EventKind = 'ui' THEN Endpoints ELSE 0 END) AS UI_Endpoints, 

        SUM(CASE WHEN EventKind = 'ui-images' THEN Events ELSE 0 END) AS Images_Events, 
        SUM(CASE WHEN EventKind = 'ui-images' THEN Sessions ELSE 0 END) AS Images_Sessions, 
        SUM(CASE WHEN EventKind = 'ui-images' THEN Endpoints ELSE 0 END) AS Images_Endpoints, 

        SUM(CASE WHEN EventKind = 'fam' THEN Events ELSE 0 END) AS FAM_Events, 
        SUM(CASE WHEN EventKind = 'fam' THEN Sessions ELSE 0 END) AS FAM_Sessions, 
        SUM(CASE WHEN EventKind = 'fam' THEN Endpoints ELSE 0 END) AS FAM_Endpoints, 

        SUM(CASE WHEN EventKind = 'cmd' THEN Events ELSE 0 END) AS CMD_Events, 
        SUM(CASE WHEN EventKind = 'cmd' THEN Sessions ELSE 0 END) AS CMD_Sessions, 
        SUM(CASE WHEN EventKind = 'cmd' THEN Endpoints ELSE 0 END) AS CMD_Endpoints
    FROM    #events e 
    GROUP BY Minute
)
SELECT  e.*,
        uiSess.Sessios AS UI_SessionsDaily, 
        uiSess.Endpoints AS UI_EndpointsDaily, 
        cmdSess.Sessios AS CMD_SessionsDaily, 
        cmdSess.Endpoints AS CMD_EndpointsDaily 
FROM    events e 
        OUTER APPLY (SELECT * FROM #sessions WHERE EventKind = 'ui-daily') uiSess
        OUTER APPLY (SELECT * FROM #sessions WHERE EventKind = 'cmd-daily')  cmdSess 



