SET NOCOUNT ON;



if object_id('tempdb..#dm_db_index_operational_stats_delta') is not null
	drop table #dm_db_index_operational_stats_delta;



declare @objectsToCheck table(object_id int)

insert into @objectsToCheck
select top 100 ps.object_id from sys.dm_db_partition_stats ps
	join sys.tables t
		on t.object_id = ps.object_id
order by row_count desc


declare @dbName sysname, 
		@objName sysname, 
		@secsToRun int, 
		@schema sysname,
		@add int, 
		@dbId int


SELECT	@dbName = db_name(),
		@dbId = DB_ID(),
		@secsToRun = 10,
		@dbName = DB_NAME(),
		@add = @secsToRun


select	
	database_id,
	object_id,
	index_id,
	partition_number,
	leaf_insert_count,
	leaf_delete_count,
	leaf_update_count,
	leaf_ghost_count,
	nonleaf_insert_count,
	nonleaf_delete_count,
	nonleaf_update_count,
	leaf_allocation_count,
	nonleaf_allocation_count,
	leaf_page_merge_count,
	nonleaf_page_merge_count,
	range_scan_count,
	singleton_lookup_count,
	forwarded_fetch_count,
	lob_fetch_in_pages,
	lob_fetch_in_bytes,
	lob_orphan_create_count,
	lob_orphan_insert_count,
	row_overflow_fetch_in_pages,
	row_overflow_fetch_in_bytes,
	column_value_push_off_row_count,
	column_value_pull_in_row_count,
	row_lock_count,
	row_lock_wait_count,
	row_lock_wait_in_ms,
	page_lock_count,
	page_lock_wait_count,
	page_lock_wait_in_ms,
	index_lock_promotion_attempt_count,
	index_lock_promotion_count,
	page_latch_wait_count,
	page_latch_wait_in_ms,
	page_io_latch_wait_count,
	page_io_latch_wait_in_ms
INTO	#dm_db_index_operational_stats_delta
from	sys.dm_db_index_operational_stats(@dbId,null,null,null)
where	object_id in (select object_id from @objectsToCheck)

alter table #dm_db_index_operational_stats_delta add date datetime default getdate()

declare @start datetime, @end datetime
set @start = GETDATE()
set @end = DATEADD(second,@add,@start)
while @start <= @end
begin
insert into #dm_db_index_operational_stats_delta
(
	database_id,
	object_id,
	index_id,
	partition_number,
	leaf_insert_count,
	leaf_delete_count,
	leaf_update_count,
	leaf_ghost_count,
	nonleaf_insert_count,
	nonleaf_delete_count,
	nonleaf_update_count,
	leaf_allocation_count,
	nonleaf_allocation_count,
	leaf_page_merge_count,
	nonleaf_page_merge_count,
	range_scan_count,
	singleton_lookup_count,
	forwarded_fetch_count,
	lob_fetch_in_pages,
	lob_fetch_in_bytes,
	lob_orphan_create_count,
	lob_orphan_insert_count,
	row_overflow_fetch_in_pages,
	row_overflow_fetch_in_bytes,
	column_value_push_off_row_count,
	column_value_pull_in_row_count,
	row_lock_count,
	row_lock_wait_count,
	row_lock_wait_in_ms,
	page_lock_count,
	page_lock_wait_count,
	page_lock_wait_in_ms,
	index_lock_promotion_attempt_count,
	index_lock_promotion_count,
	page_latch_wait_count,
	page_latch_wait_in_ms,
	page_io_latch_wait_count,
	page_io_latch_wait_in_ms
--tree_page_latch_wait_count,
--tree_page_latch_wait_in_ms,
--tree_page_io_latch_wait_count,
--tree_page_io_latch_wait_in_ms,
--page_compression_attempt_count,
--page_compression_success_count
)
select  
	database_id,
	object_id,
	index_id,
	partition_number,
	leaf_insert_count,
	leaf_delete_count,
	leaf_update_count,
	leaf_ghost_count,
	nonleaf_insert_count,
	nonleaf_delete_count,
	nonleaf_update_count,
	leaf_allocation_count,
	nonleaf_allocation_count,
	leaf_page_merge_count,
	nonleaf_page_merge_count,
	range_scan_count,
	singleton_lookup_count,
	forwarded_fetch_count,
	lob_fetch_in_pages,
	lob_fetch_in_bytes,
	lob_orphan_create_count,
	lob_orphan_insert_count,
	row_overflow_fetch_in_pages,
	row_overflow_fetch_in_bytes,
	column_value_push_off_row_count,
	column_value_pull_in_row_count,
	row_lock_count,
	row_lock_wait_count,
	row_lock_wait_in_ms,
	page_lock_count,
	page_lock_wait_count,
	page_lock_wait_in_ms,
	index_lock_promotion_attempt_count,
	index_lock_promotion_count,
	page_latch_wait_count,
	page_latch_wait_in_ms,
	page_io_latch_wait_count,
	page_io_latch_wait_in_ms 
from sys.dm_db_index_operational_stats(@dbId,null,null,null) o 
where object_id in (select object_id from @objectsToCheck)
waitfor delay '00:00:01'
set @start = dateadd(second,1,@start)
end;

with index_operational_stats_delta 
as
(
select row_number()over(partition by database_id,object_id,index_id,partition_number
 order by database_id,object_id,index_id,partition_number, date desc) as rn, *
from #dm_db_index_operational_stats_delta c
)
select [current].date,
	object_name([current].object_id) as TableName,
	[current].index_id,
	isnull(i.name,'Heap') as IndexName,
	[current].partition_number,
	isnull(datediff(second,prev.date, [current].date),1) as sample_sec_delta,
	[current].leaf_insert_count - prev.leaf_insert_count as leaf_insert_count_delta,
	[current].leaf_delete_count - prev.leaf_delete_count as leaf_delete_count_delta,
	[current].leaf_update_count - prev.leaf_update_count as leaf_update_count_delta,
	[current].leaf_ghost_count - prev.leaf_ghost_count as leaf_ghost_count_delta,
	[current].nonleaf_insert_count - prev.nonleaf_insert_count as nonleaf_insert_count_delta,
	[current].nonleaf_delete_count - prev.nonleaf_delete_count as nonleaf_delete_count_delta,
	[current].nonleaf_update_count - prev.nonleaf_update_count as nonleaf_update_count_delta,
	[current].leaf_allocation_count - prev.leaf_allocation_count as leaf_allocation_count_delta,
	[current].nonleaf_allocation_count - prev.nonleaf_allocation_count as nonleaf_allocation_count_delta,
	[current].leaf_page_merge_count - prev.leaf_page_merge_count as nleaf_page_merge_count_delta,
	[current].nonleaf_page_merge_count - prev.nonleaf_page_merge_count as nonleaf_page_merge_count_delta,
	[current].range_scan_count - prev.range_scan_count as range_scan_count_delta,
	[current].singleton_lookup_count - prev.singleton_lookup_count as singleton_lookup_count_delta,
	[current].forwarded_fetch_count - prev.forwarded_fetch_count as forwarded_fetch_count_delta,
	[current].lob_fetch_in_pages - prev.lob_fetch_in_pages as lob_fetch_in_pages_delta,
	[current].lob_fetch_in_bytes - prev.lob_fetch_in_bytes as lob_fetch_in_bytes_delta,
	[current].lob_orphan_create_count - prev.lob_orphan_create_count as lob_orphan_create_count_delta,
	[current].lob_orphan_insert_count - prev.lob_orphan_insert_count as lob_orphan_insert_count_delta,
	[current].row_overflow_fetch_in_pages - prev.row_overflow_fetch_in_pages as row_overflow_fetch_in_pages_delta,
	[current].row_overflow_fetch_in_bytes - prev.row_overflow_fetch_in_bytes as row_overflow_fetch_in_bytes_delta,
	[current].column_value_push_off_row_count - prev.column_value_push_off_row_count as column_value_push_off_row_count_delta,
	[current].column_value_pull_in_row_count - prev.column_value_pull_in_row_count as column_value_pull_in_row_count_delta,
	[current].row_lock_count - prev.row_lock_count as row_lock_count_delta,
	[current].row_lock_wait_count - prev.row_lock_wait_count as row_lock_wait_count_delta,
	[current].row_lock_wait_in_ms - prev.row_lock_wait_in_ms as row_lock_wait_in_ms_delta,
	[current].page_lock_count - prev.page_lock_count as page_lock_count_delta,
	[current].page_lock_wait_count - prev.page_lock_wait_count as page_lock_wait_count_delta,
	[current].page_lock_wait_in_ms - prev.page_lock_wait_in_ms as page_lock_wait_in_ms_delta,
	[current].index_lock_promotion_attempt_count - prev.index_lock_promotion_attempt_count as index_lock_promotion_attempt_count_delta,
	[current].index_lock_promotion_count - prev.index_lock_promotion_count as index_lock_promotion_count_delta,
	[current].page_latch_wait_count - prev.page_latch_wait_count as page_latch_wait_count_delta,
	[current].page_latch_wait_in_ms - prev.page_latch_wait_in_ms as page_latch_wait_in_ms_delta,
	[current].page_io_latch_wait_count - prev.page_io_latch_wait_count as page_io_latch_wait_count_delta,
	[current].page_io_latch_wait_in_ms - prev.page_io_latch_wait_in_ms as page_io_latch_wait_in_ms_delta--,
	--[current].tree_page_latch_wait_count - prev.tree_page_latch_wait_count as tree_page_latch_wait_count_delta,
	--[current].tree_page_latch_wait_in_ms - prev.tree_page_latch_wait_in_ms as tree_page_latch_wait_in_ms_delta,
	--[current].tree_page_io_latch_wait_count - prev.tree_page_io_latch_wait_count as tree_page_io_latch_wait_count_delta,
	--[current].tree_page_io_latch_wait_in_ms - prev.tree_page_io_latch_wait_in_ms as tree_page_io_latch_wait_in_ms_delta,
	--[current].page_compression_attempt_count - prev.page_compression_attempt_count as page_compression_attempt_count_delta,
	--[current].page_compression_success_count - prev.page_compression_success_count as page_compression_success_count_delta
from index_operational_stats_delta [current]
	join index_operational_stats_delta [prev]
		on [current].rn + 1 = [prev].rn
			and [current].database_id = prev.database_id
			and [current].object_id = prev.object_id
			and [current].index_id = prev.index_id
			and [current].partition_number = prev.partition_number
	left join sys.indexes i 
		on i.object_id = [current].object_id
			and i.index_id = [current].index_id
order by  [current].object_id,  [current].index_id,  [current].partition_number,  [current].date desc
	                                                                                                                                                                                                                    